#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import json
import configparser
import pathlib
import csv
import traceback
import requests
import urllib3
from urllib3.exceptions import InsecureRequestWarning
urllib3.disable_warnings(InsecureRequestWarning)


class APIUtils:
    def __init__(self, config_file_path):
        ''' get configuration
        '''
        # path check and resolve (config.ini file)
        p_abs = pathlib.Path(config_file_path)
        if(p_abs.is_absolute()):
            config_path = p_abs
        else:
            config_path = p_abs.resolve()
        print("read config: " + str(config_path))

        if not pathlib.Path.exists(config_path):
            raise Exception(
                "[Error] config.iniのファイルパスが存在しません。ファイルの絶対パスを確認してください。")

        # read config
        try:
            config = configparser.ConfigParser()
            config.read(config_path)
            self.DS_api_key = config['Credentials']['DS_api_key']
            self.C1WS_api_key = config['Credentials']['C1WS_api_key']
            self.DSM_url = config['Credentials']['DSM_url']
            self.workloadSecurityUrl = config['Credentials']['workloadSecurityUrl']
            self.workloadSecurityCA = config['Credentials']['workloadSecurityCA']
            self.policy_migration = config['Credentials']['policy_migration']
            self.target_computers_file_path = config['Credentials']['target_computers_file_path']
            self.proxy_manager_hostname = config['Credentials']['proxy_manager_hostname']
            self.proxy_manager_port = config['Credentials']['proxy_manager_port']
            self.proxy_manager_user = config['Credentials']['proxy_manager_user']
            self.proxy_manager_password = config['Credentials']['proxy_manager_password']
            self.proxy_relay_hostname = config['Credentials']['proxy_relay_hostname']
            self.proxy_relay_port = config['Credentials']['proxy_relay_port']
            self.proxy_relay_user = config['Credentials']['proxy_relay_user']
            self.proxy_relay_password = config['Credentials']['proxy_relay_password']
        except:
            traceback.print_exc()
            raise Exception("[Error] could not read configuration file")
        # set REST API request headers
        self.DS_header = {
            "api-version": "v1",
            "Content-Type": "application/json",
            "api-secret-key": self.DS_api_key,
        }

    def clear_workload_security_links(self):
        link_id_lists = []
        url = self.DSM_url + "/api/workloadsecuritylinks"
        # get Workload Security Links
        try:
            response = requests.get(
                url, headers=self.DS_header, verify=False)
            print("list_workload_security_links: " +
                  str(response.status_code))
            link_lists = json.loads(response.text)["workloadSecurityLinks"]
            for link in link_lists:
                link_id_lists.append(str(link["ID"]))
        except requests.exceptions.RequestException as e:
            raise Exception(
                "[Error] list_workload_security_links: リクエストに失敗しました。DSMとの接続を確認してください。", e)
        except KeyError as e:
            raise Exception(
                "[Error] list_workload_security_links: リクエストに必要な情報が不足しています。config.iniの設定項目を確認してください。", e)
        except:
            traceback.print_exc()
            raise Exception("[Error] list_workload_security_links")

        # delete Workload Security Link
        if link_id_lists:
            for link_id in link_id_lists:
                url = self.DSM_url + "/api/workloadsecuritylinks/" + link_id
                try:
                    response = requests.delete(
                        url, headers=self.DS_header, verify=False)
                    print("delete_workload_security_links: " +
                          str(response.status_code))
                except requests.exceptions.RequestException as e:
                    raise Exception(
                        "[Error] delete_workload_security_links: リクエストに失敗しました。DSMとの接続を確認してください。", e)
                except:
                    traceback.print_exc()
                    raise Exception("[Error] delete_workload_security_links")
        else:
            print("delete_workload_security_links: Pass")

    def create_workload_security_links(self):
        url = self.DSM_url + "/api/workloadsecuritylinks"
        body = {
            "workloadSecurityUrl": self.workloadSecurityUrl,
            "workloadSecurityAPIKey": self.C1WS_api_key,
            "workloadSecurityCA": self.workloadSecurityCA
        }
        try:
            response = requests.post(
                url, headers=self.DS_header, data=json.dumps(body).replace(r"\\n", r"\n"), verify=False)
            print("create_workload_security_links: " +
                  str(response.status_code) + "\t" + response.text)
            link_id = json.loads(response.text)["ID"]
            time.sleep(1)
            return link_id
        except requests.exceptions.RequestException as e:
            raise Exception(
                "[Error] create_workload_security_links: リクエストに失敗しました。DSMとの接続を確認してください。", e)
        except KeyError as e:
            raise Exception(
                "[Error] create_workload_security_links: リクエストに必要な情報が不足しています。config.iniの設定項目を確認してください。", e)
        except:
            traceback.print_exc()
            raise Exception("[Error] create_workload_security_links")

    def create_policy_migration_task(self):
        url = self.DSM_url + "/api/policymigrationtasks"
        try:
            response = requests.post(
                url, headers=self.DS_header, verify=False)
            print("create_policy_migration_task: " +
                  str(response.status_code) + "\t" + response.text)
            policy_migration_task_id = str(json.loads(response.text)["ID"])
            return policy_migration_task_id
        except requests.exceptions.RequestException as e:
            raise Exception(
                "[Error] create_policy_migration_task: リクエストに失敗しました。DSMとの接続を確認してください。", e)
        except:
            traceback.print_exc()
            raise Exception("[Error] create_policy_migration_task")

    def describe_policy_migration_task(self, policy_migration_task_id):
        url = self.DSM_url + "/api/policymigrationtasks/" + policy_migration_task_id
        policies = {}
        # support policy "None"
        policies[""] = ""
        status = ""

        # wait for the policy migration to complete
        for i in range(5):
            if status == "complete":
                break
            elif status == "failed":
                raise Exception(
                    "[Error] describe_policy_migration_task: policy_migration failed")
            else:
                print("describe_policy_migration_task: ポリシー移行中のため 1分待機します")
                time.sleep(60)
                try:
                    response = requests.get(
                        url, headers=self.DS_header, verify=False)
                    status = json.loads(response.text)["status"]
                except requests.exceptions.RequestException as e:
                    raise Exception(
                        "[Error] describe_policy_migration_task: リクエストに失敗しました。DSMとの接続を確認してください。", e)
                except:
                    traceback.print_exc()
                    raise Exception("[Error] describe_policy_migration_task")
                i += 1

        # get policyMappings information
        if i == 5:
            raise Exception(
                "[Error] describe_policy_migration_task: リクエストに失敗しました。DSMとの接続を確認してください。")
        else:
            mappings = json.loads(response.text)["policyMappings"]
            for mapping in mappings:
                policies[mapping["migrateFrom"]["policyID"]
                         ] = mapping["migrateTo"]["policyID"]
            return policies

    def get_target_computers(self):
        # path check and resolve (target_computers_file_path)
        p_target_abs = pathlib.Path(self.target_computers_file_path)
        if(p_target_abs.is_absolute()):
            self.target_hosts_file_path = p_target_abs
        else:
            self.target_hosts_file_path = p_target_abs.resolve()

        if not pathlib.Path.exists(self.target_hosts_file_path):
            raise Exception(
                "[Error] コンピュータリストのファイルパスが存在しません。ファイルの絶対パスを確認してください。")

        # parse Host GUID
        try:
            csv_file = open(self.target_hosts_file_path, "r",
                            encoding="ms932", errors="", newline="")
            f = csv.reader(csv_file, delimiter=",", doublequote=True,
                           lineterminator="\r\n", quotechar='"', skipinitialspace=True)
            header = next(f)
            index = header.index('Host GUID')
            target_computers = []
            for row in f:
                target_computers.append(row[index])
            return target_computers
        except LookupError as e:
            raise Exception(
                "[Error] get_target_computers: コンピュータリストのファイルにHost GUIDが含まれていません。ファイルを確認してください。", e)
        except:
            traceback.print_exc()
            raise Exception("[Error] get_target_computers")

    def list_computer_policy_id(self, target_computers):
        id_lists = {}
        if target_computers:
            # search computers
            url = self.DSM_url + "/api/computers/search"
            try:
                for target in target_computers:
                    body = {
                        "maxItems": 1,
                        "searchCriteria": [
                            {
                                "fieldName": "hostGUID",
                                "stringTest": "equal",
                                "stringValue": target
                            }
                        ],
                        "sortByObjectID": True
                    }
                    response = requests.post(
                        url, headers=self.DS_header, data=json.dumps(body), verify=False)
                    computers = json.loads(response.text)["computers"]
                    print("list_computer_policy_id Search " + target + ": " +
                          str(response.status_code))
                    if computers[0]["computerStatus"]["agentStatus"] == "active":
                        if "policyID" in computers[0]:
                            id_lists[computers[0]["ID"]
                                     ] = computers[0]["policyID"]
                        else:
                            id_lists[computers[0]["ID"]] = ""
                    else:
                        print("list_computer_policy_id Search: skip " +
                              computers[0]["hostName"] + "(" + computers[0]["computerStatus"]["agentStatus"] + ")")
                return id_lists
            except requests.exceptions.RequestException as e:
                raise Exception(
                    "[Error] list_computer_policy_id SEARCH: リクエストに失敗しました。DSMとの接続を確認してください。", e)
            except:
                traceback.print_exc()
                raise Exception("[Error] list_computer_policy_id SEARCH")
        else:
            # list all computers
            url = self.DSM_url + "/api/computers"
            try:
                response = requests.get(
                    url, headers=self.DS_header, verify=False)
                computers = json.loads(response.text)["computers"]
                print("list_computer_policy_id All: " +
                      str(response.status_code))
                for computer in computers:
                    if computer["computerStatus"]["agentStatus"] == "active":
                        if "policyID" in computer:
                            id_lists[computer["ID"]] = computer["policyID"]
                        else:
                            id_lists[computer["ID"]] = ""
                    else:
                        print("list_computer_policy_id All: skip " +
                              computer["hostName"] + "(" + computer["computerStatus"]["agentStatus"] + ")")
                return id_lists
            except requests.exceptions.RequestException as e:
                raise Exception(
                    "[Error] list_computer_policy_id ALL: リクエストに失敗しました。DSMとの接続を確認してください。", e)
            except:
                traceback.print_exc()
                raise Exception("[Error] list_computer_policy_id ALL")

    def create_computer_move_task(self, policies, id_lists):
        url = self.DSM_url + "/api/computermovetasks"
        for computer_id in id_lists:
            body = {"computerID": computer_id}
            if policies:
                body["workloadSecurityPolicyID"] = policies[id_lists[computer_id]]
            if self.proxy_manager_hostname:
                body["workloadSecurityProxy"] = {
                    "host": self.proxy_manager_hostname,
                    "port": self.proxy_manager_port,
                    "user": self.proxy_manager_user,
                    "password": self.proxy_manager_password
                }
            if self.proxy_relay_hostname:
                body["workloadSecurityRelayProxy"] = {
                    "host": self.proxy_relay_hostname,
                    "port": self.proxy_relay_port,
                    "user": self.proxy_relay_user,
                    "password": self.proxy_relay_password
                }
            try:
                response = requests.post(
                    url, headers=self.DS_header, data=json.dumps(body), verify=False)
                print("create_computer_move_task: " +
                      str(response.status_code) + "\t" + response.text)
                time.sleep(1)
            except requests.exceptions.RequestException as e:
                raise Exception(
                    "[Error] create_computer_move_task: リクエストに失敗しました。DSMとの接続を確認してください。", e)
            except KeyError as e:
                raise Exception(
                    "[Error] create_computer_move_task: リクエストに必要な情報が不足しています。config.iniの設定項目を確認してください。", e)
            except:
                traceback.print_exc()
                raise Exception("[Error] create_computer_move_task")
